#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   orm.py    
@Contact :   wangxin01@loongson.cn
@License :   (C)Copyright 2021

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
21-5-25 上午7:31   wangxin    1.0         None
'''

from peewee import *

from src.const.const import sys_const
from src.exception.BusinessException import BusinessException
from src.message.message import E0001
from src.utils.timeutils import get_current_datetime

db = SqliteDatabase(sys_const.SQLITE_DB_PATH)

import logging

logger = logging.getLogger('peewee')
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


class BaseModel(Model):
    class Meta:
        database = db


class LocationInfo(BaseModel):
    location_id = PrimaryKeyField(null=False)
    location_name_en = CharField()
    location_name = CharField(null=False)
    country_code = CharField()
    country_en = CharField()
    country_name = CharField()
    adm1_en = CharField()
    adm1 = CharField()
    adm2_en = CharField()
    adm2 = CharField()
    latitude = CharField()
    longitude = CharField()
    ad_code = CharField()

    class Meta:
        order_by = ('location_id',)
        db_table = 'location_info'


class LocationCollection(BaseModel):
    ip = CharField(null=True)  # IP地址
    id = CharField(primary_key=True)  # 地区ID
    lat = CharField(null=True)  # # 纬度
    lng = CharField(null=True)  # 经度
    nation = CharField(null=True)  # 国家地区
    province = CharField(null=True)  # 省份
    city = CharField(null=True)  # 城市
    district = CharField(null=True)  # 区
    deleted_flag = BooleanField(null=False)  # 删除Flag
    update_time = TimeField()  # 更新时间

    class Meta:
        order_by = ('update_time',)
        db_table = 'location_collection'


# 精确搜索城市信息
def get_location_info_by_location_name(location_name):
    try:
        return LocationInfo.get(LocationInfo.location_name == location_name)
    except:
        raise BusinessException(E0001)


# 跟据地区ID查找地区信息
def get_location_info_by_location_id(location_id):
    return LocationInfo.get(LocationInfo.location_id == location_id)


# 取得收藏表前10条数据
def get_location_collection(limitNum=10):
    try:
        return LocationCollection.select().where(LocationCollection.deleted_flag == 0).order_by(
            # 根据时间戳倒叙排列使得新添加的数据排列在第一位
            LocationCollection.update_time.desc()).limit(limitNum)
    except:
        raise


# 取得收藏表数据个数
def get_location_collection_count():
    try:
        return LocationCollection.select().where(LocationCollection.deleted_flag == 0).count()
    except:
        raise


# 保存数据
@db.atomic()
def upsert_location_collect(location):
    try:
        result = (LocationCollection
                  .insert(ip=location.ip,  # IP地址
                          id=location.id,  # 地区ID
                          lat=location.lat,  # # 纬度
                          lng=location.lng,  # 经度
                          nation=location.nation,  # 国家地区
                          province=location.province,  # 省份
                          city=location.city,  # 城市
                          district=location.district,  # 区
                          deleted_flag=location.deleted_flag,  # 删除Flag
                          # 收藏列表更新时间数据时改为时间戳记录
                          update_time=get_current_datetime().timestamp() * 1000)  # 更新时间
                  .on_conflict('replace')
                  .execute())
    except:
        raise


# 该数据是否存在
def location_collect_exists(location_id):
    query = LocationCollection.select().where(LocationCollection.id == location_id)
    return query.exists()


# 取得该收藏天气信息
def get_location_collection_data(location_id):
    return LocationCollection.get(LocationCollection.id == location_id)


# 精确搜索城市信息
def get_location_data_by_location_name(name):
    return LocationInfo.select().where(LocationInfo.location_name == name)


# 模糊搜索城市信息
def get_location_data_by_location_name_fuzzy(name):
    return LocationInfo.select().where(LocationInfo.location_name.contains(name))


# 精确搜索二级行政单位取得城市信息
def get_location_data_by_adm2(adm2):
    return LocationInfo.select().where(LocationInfo.adm2 == adm2)


# 模糊搜索二级行政单位取得二级行政单位
def get_location_data_by_adm2_fuzzy(adm2):
    return LocationInfo.select().where(LocationInfo.adm2.contains(adm2)).group_by(LocationInfo.adm2)


# 精确搜索一级行政单位取得二级行政单位
def get_location_data_by_adm1(adm1):
    return LocationInfo.select().where(LocationInfo.adm1 == adm1).group_by(LocationInfo.adm2)


# 模糊搜索一级行政单位取得一级行政单位
def get_location_data_by_adm1_fuzzy(adm1):
    return LocationInfo.select().where(LocationInfo.adm1.contains(adm1)).group_by(LocationInfo.adm1)
