import os
import sys

from PyQt5 import QtCore, QtWidgets, QtGui, Qt
from PyQt5.QtWidgets import QApplication

PathRole = QtCore.Qt.UserRole + 1
ImageRequestedRole = PathRole + 1


class ImageDelegate(QtWidgets.QStyledItemDelegate):
    def initStyleOption(self, opt, index):
        super().initStyleOption(opt, index)
        if index.data(PathRole):
            opt.features |= opt.HasDecoration
            opt.decorationSize = QtCore.QSize(180, 160)


class MyQListWidget(QtWidgets.QListWidget):
    def __init__(self):
        super(MyQListWidget, self).__init__()
        path = '../images'
        self.setFlow(QtWidgets.QListView.LeftToRight)
        self.setIconSize(QtCore.QSize(180, 160))
        self.setResizeMode(Qt.QListWidget.Adjust)

        for fp in os.listdir(path):
            imagePath = os.path.join(path, fp)
            item = QtWidgets.QListWidgetItem()
            if os.path.isfile(imagePath):
                item.setData(PathRole, imagePath)
            self.addItem(item)

        self.imageDelegate = ImageDelegate(self)
        self.setItemDelegate(self.imageDelegate)
        self.imageQueue = []
        self.loadTimer = QtCore.QTimer(
            interval=25, timeout=self.loadImage, singleShot=True)
        self.lazyTimer = QtCore.QTimer(
            interval=100, timeout=self.lazyLoadImage, singleShot=True)
        self.lazyIndex = 0

        self.horizontalScrollBar().valueChanged.connect(self.checkVisible)
        self.horizontalScrollBar().rangeChanged.connect(self.checkVisible)

    def checkVisible(self):
        start = self.indexAt(QtCore.QPoint()).row()
        end = self.indexAt(self.viewport().rect().bottomRight()).row()
        if end < 0:
            end = start
        model = self.model()
        for row in range(start, end + 1):
            index = model.index(row, 0)
            if not index.data(ImageRequestedRole) and index.data(PathRole):
                with QtCore.QSignalBlocker(model):
                    model.setData(index, True, ImageRequestedRole)
                self.imageQueue.append(index)
        if self.imageQueue and not self.loadTimer.isActive():
            self.loadTimer.start()

    def requestImage(self, index):
        with QtCore.QSignalBlocker(self.model()):
            self.model().setData(index, True, ImageRequestedRole)
        self.imageQueue.append(index)
        if not self.loadTimer.isActive():
            self.loadTimer.start()

    def loadImage(self):
        if not self.imageQueue:
            return
        index = self.imageQueue.pop()
        image = QtGui.QPixmap(index.data(PathRole))
        if not image.isNull():
            self.model().setData(
                index,
                image.scaled(self.iconSize(), QtCore.Qt.KeepAspectRatio),
                QtCore.Qt.DecorationRole
            )
        if self.imageQueue:
            self.loadTimer.start()
        else:
            self.lazyTimer.start()

    def lazyLoadImage(self):
        self.lazyIndex += 1
        if self.lazyIndex >= self.count():
            return
        index = self.model().index(self.lazyIndex, 0)
        if not index.data(ImageRequestedRole) and index.data(PathRole):
            with QtCore.QSignalBlocker(self.model()):
                self.model().setData(index, True, ImageRequestedRole)
            image = QtGui.QPixmap(index.data(PathRole))
            if not image.isNull():
                self.model().setData(
                    index,
                    image.scaled(self.iconSize(), QtCore.Qt.KeepAspectRatio),
                    QtCore.Qt.DecorationRole
                )
        else:
            self.lazyLoadImage()
            return
        if not self.imageQueue:
            self.lazyTimer.start()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    form = MyQListWidget()
    form.show()
    sys.exit(app.exec_())
